%
%   This function solves the set of nonlinear equations
%
%       G(alpha) = 0
%
%   using a homotopy continuation method. The initial point 
%    is taken as the central solution. 
%
%   Input: interdata - an interpolation data set
%
%   Output: interpolant - an interpolant data set
%
function interpolant = homotopy(interdata)

%
%   Parameters
%
stepmu = 0.1;           %   tolerance in predictor step
maxiter = 50;           %   maximal number of main iiterations  
errorbound = 1e-8;      %   maximal 1-norm of LHS, final
errorboundway = 1e-5;   %   maximal 1-norm of LHS, on the way
digit = 8;              %   for display
displayrate = 1;        %   How often to display

%
%   Make data self-conjugated, move first interpolation point to zero
%    and check that data is valid.
%
interdataold = interdata;
[ interdata, tozerotrans ] = DataValid(interdata);
if isempty(interdata.dpoly)
    interdata.dpoly = (GetDissPoly(interdata.tau))';
end

%
%   Determine central solution
%
alphavec = [];
alphaveccorr = [];
alphavecpred = [];
alpha = squeeze(GetCentral(interdata));
K = GetK(interdata);

%
%   Determine the coefficinets mu at the central and 
%    desired solutions
%
mu0 = 1/2 * GetS(interdata.tau) * interdata.tau';
mu1 = interdata.dpoly'; %1/2 * GetS(rho) * rho';
kappa = mu0(1) / mu1(1);
mu1 = mu1 * kappa;
mu0 = mu0(:); mu1 = mu1(:);
%
%   Check whether we are looking for the central solution
%
if norm(mu1-mu0) < 1e-10
   disp('This is the central solution.')
   error = EvaluateG(K,alpha,mu0,mu1,1);
   if error > errorbound,
       disp('Solution might be inaccurate'), error
   end
   interpolant = GetInterpolant(alpha,K);
   interpolant = BilinearDomain(tozerotrans,interpolant);
   return
end

%
%   Gradient at the central solution
%
gradG = GetGradient(interdata,alpha);

%
%   Compute dG/dnu
%
dG_dnu = mu0 - mu1;

disp('--------------------------------------------------------')
disp('    Nevanlinna-Pick solver using a continuation method  ')
disp('--------------------------------------------------------')
disp('                    Corrector step                      ')
disp('     nu       iteration      equation error             ')
disp('--------------------------------------------------------')

nu = 0; k = 0;
while nu ~= 1
    k = k + 1;
    
    alphavec(:,end+1) = alpha;
    alphavecpred(k,:,1) = alpha;
    %
    %   Predictor step:
    %
    %    Calculate the direction for a step in nu
    %
    dalpha_dnu = gradG \ dG_dnu;
    
    %
    %   Determine the step size for nu
    %
    dnu = FindStepSize(alpha,dalpha_dnu,K,stepmu);
    if nu + dnu > 1,
        dnu = 1 - nu;
    end
    
    %
    %   We need to stay within D
    %
    itertmp = 0;
    while IsInD(K,alpha+dnu*dalpha_dnu) ~= 1
        dnu = dnu / 2;
        itertmp = itertmp + 1;
        if itertmp > maxiter
            disp('Broken in D test')
            break
        end
    end
    nu = nu + dnu;
    disp(num2str(nu,digit))

    %
    %   Corrector step:
    %
    %       Update the polynomial alpha check error.
    %
    alpha = alpha - dnu*dalpha_dnu;
    alphavecpred(k,:,2) = alpha;
    
    iter = 0;
    errorold = 0;
    gradG = GetGradient(interdata,alpha);
    Gvalue = EvaluateG(K,alpha,mu0,mu1,nu);
    
    alphaveccorr(k,:,1) = alpha;
    kcorr = 1;
    while (nu < 1 | max(abs(Gvalue)) > errorbound) & (nu == 1 | max(abs(Gvalue)) > errorboundway)
        dalpha = - gradG \ Gvalue;
        
        %
        %   We check that we stay in D.
        %
        lambda = 1;
%        while IsInD(K,alpha+lambda*dalpha) ~= 1     
%            lambda = lambda / 2;
%        end

        %
        %   Update the solution and check interpolation error
        %
        alpha = alpha + lambda*dalpha;
        gradG = GetGradient(interdata,alpha);
        Gvalue = EvaluateG(K,alpha,mu0,mu1,nu);
        error = max(Gvalue);
        
        kcorr = kcorr + 1;
        alphaveccorr(k,:,kcorr) = alpha;
        
        iter = iter + 1;
        if mod(iter,displayrate) == 0
            
            errorold = error; 
            
            iterstr = num2str(iter);
            errorstr = num2str(error,digit);
            disp([blanks(20 - length(iterstr)) iterstr, ...
                        blanks(20 - length(errorstr)) errorstr])
        end
        if iter > maxiter
            disp('Broken corrector step')
            break
        end
    end
end
alphavec(:,end+1) = alpha;

disp('--------------------------------------------------')

interpolant = GetInterpolant(alpha,K);
interpolant = BilinearDomain(tozerotrans,interpolant);

%
%   A local function for evaluating the LHS of the nonlinear equation
%
function G = EvaluateG(K,alpha,mu0,mu1,nu)

% Note: we can probably speed this up by storing S_alpha*K*alpha
G = GetS(alpha)*K*alpha - (1-nu)*mu0 - nu*mu1;